#include <cmath>
using std::fmod;

#include "timers.hh"

unsigned timer::Advance(double length, double add)
{
	double sum = (add + timefraction) / length;
	
	unsigned ticks = static_cast<unsigned> (sum);
	
	timefraction = length * (sum - ticks);
	
	return ticks;
}

void timer::Run(double seconds, double length)
{
	if(active)
	{
		unsigned n = Advance(length, seconds);
		if(n)
		{
			unsigned newvalue = value + n;
			if(irqs)hit = newvalue > 255;
			value = newvalue & 255;
		}
	}
}

void timer::Activate()
{
	active = true;
}

void timer::Deactivate()
{
	active = false;
	timefraction = 0.0;
}

void timer::Set(unsigned char nv)
{
	value = nv;
}

void Timers::Run(double seconds)
{
	timer1.Run(seconds,  80E-6);
	timer2.Run(seconds, 320E-6);
}

const unsigned char Timers::ReadStatus() const
{
	unsigned char t = 0;
	if(timer1.hit) t |= 0x40;
	if(timer2.hit) t |= 0x20;
	if(t)t |= 0x80;
	return t;
}

void Timers::WriteTimer1(unsigned char value)
{
	timer1.Set(value);
}

void Timers::WriteTimer2(unsigned char value)
{
	timer2.Set(value);
}

void Timers::WriteReg4(unsigned char value)
{
	if(value & 0x80)
	{
		timer1.hit = false;
		timer2.hit = false;
	}
	else
	{
		if(value & 1)timer1.Activate(); else timer1.Deactivate();
		if(value & 2)timer2.Activate(); else timer2.Deactivate();
		
		timer1.irqs = !(value & 0x40);
		timer2.irqs = !(value & 0x20);
	}
}
